#version 330

layout (std140) uniform Materials
{
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

layout (std140) uniform Lights
{
	vec3 l_dir;		// camera space
};

in Data
{
	vec3 normal;	// camera space
	vec4 eye;		// camera space
	vec4 shadowCoord;
} DataIn;

uniform sampler2DShadow shadowUnit;

out vec4 outputF;

//----------------------------------------------------------------
// TODO:
// 1) Choose between shadow (no light / color) and per pixel lighting.
//	- use textureProj to access the shadow value.
//	- if the result is less than 1.f the point is in shadow.
// 2) Notice the sampler type, why is that?
// 3) Optionaly use some ambient light.
//----------------------------------------------------------------

void main()
{
	// Get compare result between the shadow texture and depth buffer
	// http://www.opengl.org/sdk/docs/manglsl/xhtml/textureProj.xml
	// x,y are texture coordinates, z is camera depth, w is used for perspective division


	// Set the specular term to black
	vec4 spec = vec4(0.f);

	// Normalize both input vectors
	vec3 n = normalize(DataIn.normal);
	vec3 e = normalize(vec3(DataIn.eye));

	float intensity = max(dot(n, l_dir), 0.f);

	// If the vertex is lit compute the specular color
	if(intensity > 0.f)
	{
		// Compute the half vector
		vec3 h = normalize(l_dir + e);	
		// Compute the specular term into spec
		float intSpec = max(dot(h, n), 0.f);
		spec = specular * pow(intSpec, shininess);
	}
	
	vec4 col = ambient + intensity * diffuse + spec;
	outputF = clamp(col, 0.f, 1.f);
}